//// Constantes

// Ressources plugins
const { dest, parallel, series, src, task, watch } 	= require('gulp')
				minifycss 		= require('gulp-clean-css'),
				concat  			= require('gulp-concat'),
				uglify 				= require('gulp-uglify'),
				browserSync 	= require('browser-sync').create();



//// Fonctions / sous tâches

// Copier les ressources dans le nouveau dossier
task('copy-assets', function() {
	return src( [
			// On copie tout...
			'src/**/*.*',

			// sauf le css, qui sera copié via minify css
			'!src/css/**/*.css',
			// sauf le js, qui sera copié via uglify js
			'!src/js/**/*.js'
			]
		)
		// On met dans le tuyau (c'est comme ça, chut)
		// 		   On envoie les fichiers dans le dossier build/
		.pipe(dest('build/'));
});



// Copier le html, pour les mises à jour avec browser-sync
task('copy-html', function() {
	return src( 'src/**/*.html')
		.pipe(dest('build/'))

		// On met à jour le navigateur
		.pipe(browserSync.stream());
});



// Minifier le css
// https://www.npmjs.com/package/gulp-clean-css
task('minify-css', function() {
	return src(
			[
				// On copie tout...
				'src/css/**/*.css',

				// sauf les librairies JS, qui ont leur propre tâche dédiée
				'!src/css/libraries/**/*.*'
				]
		)
		// on minifie l'ensemble des fichiers
		.pipe(minifycss())
		// on les regroupe en un seul
		.pipe(concat('main.css'))
		.pipe(dest('build/css/'))

		// On met à jour le navigateur
		.pipe(browserSync.stream());
});



// Minifier les librairies css (ca prend un peu plus de temps)
// C'est pas glamour pour le watch
task('minify-css-libs', function () {
	return src('src/css/libraries/**/*.css')
		// on minifie l'ensemble des fichiers
		.pipe(minifycss())
		// on les regroupe en un seul
		.pipe(concat('libs.css'))
		.pipe(dest('build/css/'))

		// On met à jour le navigateur
		.pipe(browserSync.stream());
});



// Minifier notre js
task('minify-js', function () {
	return src(
			[
				// On copie tout...
				'src/js/**/*.js',

				// sauf les librairies JS, qui ont leur propre tâche dédiée
				'!src/js/vendor/**/*.js'
				]
		)
		// on minifie l'ensemble des fichiers
		.pipe(uglify())
		// on les regroupe en un seul
		.pipe(concat('main.js'))
		.pipe(dest('build/js/'))

		// On met à jour le navigateur
		.pipe(browserSync.stream());
});



// Minifier les librairies js (ca prend un peu plus de temps)
// C'est pas glamour pour le watch
task('minify-js-libs', function () {
	return src('src/js/vendor/**/*.js')
	// on minifie l'ensemble des fichiers
		.pipe(uglify())
		// on les regroupe en un seul
		.pipe(concat('libs.js'))
		.pipe(dest('build/js/'))

		// On met à jour le navigateur
		.pipe(browserSync.stream());
});



// Watch / Ecouter les actions utilisateur et les répercuter sur le navigateur
// https://gulpjs.com/docs/en/api/watch
task('start-watch', function () {
	// On lanc eun serveur local js
  browserSync.init({
      server: "./build"
  });

  // On écoute les changements de fichiers html/css/js,
	const watcherHTML = watch(['src/**/*.html']);
	const watcherCSS 	= watch(['src/css/**/*.css']);
	const watcherJS 	= watch([	'src/js/**/*.js',
															'!src/js/vendor/**/*.js'
														]);

  // puis on apelle leur tâches correspondantes en cas de modifications
  watcherHTML	.on('change', parallel('copy-html'));
  watcherCSS	.on('change', parallel('minify-css'));
  watcherJS		.on('change', parallel('minify-js'));

  // watch("src/css/**/*.css", 	['minify-css']);
	//
  // watch("src/js/**/*.js", 	['minify-js']);

  // watch("src/*.html").on('change', browserSync.reload);
});



//// Tâches

// Fonction par défaut, appelée dans la console en tapant 'gulp'
//		Le premier paramètre est le nom de la tâche
//		Cette tâche appelle chaque tâche contenue dans le tableau passé en 2eme paramètre

//  	La tâche par défaut appelle la tâche copy-assets, ainsi que la tâche minifier le css
task('default', parallel('copy-assets', 'minify-css', 'minify-css-libs', 'minify-js', 'minify-js-libs'));

//// Watch, actualisation à la modification
// On éxécute la tâche par défaut avant, puis..
task('watch', series('default', 'start-watch'));
